@@ -50,6 +50,10 @@ class BaseHandler(RequestHandler):
         self.set_header("Content-Type", "application/json")
         self.write(dumps(obj))
 
+    @property
+    def cache(self):
+        return self.application.cache
+
     @property
     def db(self):
         return self.application.db
